@@ -53,7 +53,7 @@ static void cb_click(GtkComboBox *widget,gComboBox *data)
 		data->emit(SIGNAL(data->onChange));
 	}
 	
-	if (!data->_no_click && data->index() >= 0)
+	if (data->index() >= 0)
 		data->emit(SIGNAL(data->onClick));
 }
 
@@ -144,7 +144,7 @@ gComboBox::gComboBox(gContainer *parent) : gTextBox(parent, true)
 	onClick = NULL;
 	onActivate = NULL;
 	
-	_no_click = false;
+	//_no_click = false;
 	_last_key = 0;
 	_model_dirty = false;
 	sort = false;
@@ -212,11 +212,14 @@ int gComboBox::index()
 	return gtk_combo_box_get_active (GTK_COMBO_BOX(widget));
 }
 
-char* gComboBox::itemText(int ind)
+char *gComboBox::itemText(int ind)
 {
 	gTreeRow *row;
 	gTreeCell *cell;
 	
+	if (ind < 0)
+		return NULL;
+	
 	updateModel();
 	
 	char *key = indexToKey(ind);
@@ -264,7 +267,13 @@ void gComboBox::setIndex(int vl)
 		vl = -1;
 	else if (vl >= count()) 
 		return;
-		
+	
+	if (vl == index())
+	{
+		emit(SIGNAL(onClick));
+		return;
+	}
+	
 	updateModel();
 	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), vl);
 }
@@ -313,10 +322,7 @@ void gComboBox::setReadOnly(bool vl)
 		gtk_widget_show(entry);
 
 		if (count())
-		{
-			//fprintf(stderr, "setReadOnly: gTextBox::setText: %s\n", itemText(index()));
 			gTextBox::setText(itemText(index()));
-		}
 		
 		setBackground(background());
 		setForeground(foreground());
@@ -329,11 +335,20 @@ void gComboBox::setReadOnly(bool vl)
 		g_signal_connect(G_OBJECT(entry), "focus-out-event", G_CALLBACK(gcb_focus_out), (gpointer)this);
 	}
 	
-	if ( vl && entry ) 
+	if (vl && entry) 
 	{
+		char *txt = g_strdup(text());
+		
+		setIndex(-1);
 		gtk_widget_destroy(entry);
-		//gtk_container_remove(GTK_CONTAINER(widget), entry);
 		entry = NULL;
+		_button = NULL;
+		
+		_model_dirty = true;
+		updateModel();
+		
+		setText(txt);
+		g_free(txt);
 	}
 	
 	updateFocusHandler();
@@ -348,8 +363,6 @@ void gComboBox::setText(const char *vl)
 {
 	int index = find(vl);
 	
-	//fprintf(stderr, "setText: %s\n", vl);
-	
 	if (index >= 0)
 		setIndex(index);
 	else
