@@ -31,7 +31,8 @@ public:
 	virtual void afterRefresh();
 
 //"Private"
-	void updateSize(bool noresize = false);
+	void updateSize(bool adjust = false, bool noresize = false);
+	void updateLayout();
 	PangoLayout *layout;
 	int align,lay_x,lay_y;
 	unsigned markup : 1;
