@@ -74,7 +74,9 @@ static void cb_hide (GtkWidget *widget, gMainWindow *data)
 
 static gboolean win_close(GtkWidget *widget,GdkEvent  *event,gMainWindow *data)
 {
-	data->doClose();
+	if (!gMainWindow::_current || data == gMainWindow::_current)
+		data->doClose();
+	
 	return true;
 }
 
@@ -435,17 +437,14 @@ void gMainWindow::afterShow()
 
 void gMainWindow::setVisible(bool vl)
 {
+  gMainWindow *active;
+
 	if (vl)
 	{
 		emitOpen();
 		if (!opened)
 			return;
 		
-		if (isTopLevel() && !_xembed)
-		{
-			gtk_window_group_add_window(gApplication::currentGroup(), GTK_WINDOW(border));
-		}
-			
 		_not_spontaneous = !visible;
 		visible = true;
 		_hidden = false;
@@ -454,8 +453,18 @@ void gMainWindow::setVisible(bool vl)
 		{
 			if (!_title || !*_title)
 				gtk_window_set_title(GTK_WINDOW(border), gApplication::defaultTitle());
-			gtk_window_present(GTK_WINDOW(border));
+
+			if (!_xembed)
+			{
+				gtk_window_group_add_window(gApplication::currentGroup(), GTK_WINDOW(border));
+
+				active = gDesktop::activeWindow();
+				if (active && active != this)
+					gtk_window_set_transient_for(GTK_WINDOW(border), GTK_WINDOW(active->border));
+			}
+			
 			gtk_window_move(GTK_WINDOW(border), bufX, bufY);
+			gtk_window_present(GTK_WINDOW(border));
 		}
 		else 
 		{
@@ -541,7 +550,6 @@ bool gMainWindow::modal()
 void gMainWindow::showModal()
 {
   gMainWindow *save;
-  gMainWindow *active;
 	
 	if (!isTopLevel()) return;
 	if (modal()) return;
@@ -549,12 +557,10 @@ void gMainWindow::showModal()
 	save = _current;
 	_current = this;
 	
-	active = gDesktop::activeWindow();
-	if (active && active != this)
-    gtk_window_set_transient_for(GTK_WINDOW(border), GTK_WINDOW(active->border));
 	gtk_window_set_modal(GTK_WINDOW(border), true);
   
   center();
+	//show();
 
 	//fprintf(stderr, "showModal: begin %p\n", this);
 
@@ -570,7 +576,6 @@ void gMainWindow::showModal()
 		destroyNow();
 	else
 		hide();
-	
 }
 
 void gMainWindow::raise()
@@ -869,6 +874,9 @@ bool gMainWindow::doClose()
 
 	if (opened)
 	{
+		if (modal() && !gApplication::hasLoop(this))
+			return true;
+
 		_closing = true;
 		if (onClose) 
 		{
@@ -1028,7 +1036,9 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 	GtkWidget *new_border;
 	int w, h;
 	gColor fg, bg;
-	
+	int i;
+	gControl *child;
+
 	if (_xembed)
 		return;
 	
@@ -1062,6 +1072,16 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 		
 		move(x, y);
 		gtk_widget_set_size_request(border, width(), height());
+
+		// Hidden children are incorrectly shown. Fix that!
+		for (i = 0;; i++)
+		{
+			child = getControl(i);
+			if (!child)
+				break;
+			if (!child->isVisible())
+				child->setVisible(false);
+		}
 	}
 	else if (!isTopLevel() && !newpr)
 	{
