@@ -357,9 +357,7 @@ gPicture *Grab_gdkWindow(GdkWindow *win)
 
 void gMnemonic_correctText(char *st,char **buf)
 {
-	long bucle,b2;
-	long len=strlen(st);
-	long len_in=len;
+	int bucle,b2;
 	
 	if (!st || !*st)
 	{
@@ -367,6 +365,9 @@ void gMnemonic_correctText(char *st,char **buf)
 		return;
 	}
 	
+	int len = strlen(st);
+	int len_in = len;
+	
 	for (bucle=0;bucle<len_in;bucle++)
 	{
 		if (st[bucle]=='&')
@@ -419,9 +420,7 @@ void gMnemonic_correctText(char *st,char **buf)
 
 guint gMnemonic_correctMarkup(char *st,char **buf)
 {
-	long bucle,b2;
-	long len=strlen(st);
-	long len_in=len;
+	int bucle,b2;
 	guint retval=0;	
 
 	if (!st || !*st)
@@ -430,6 +429,9 @@ guint gMnemonic_correctMarkup(char *st,char **buf)
 		return retval;
 	}
 	
+	int len = strlen(st);
+	int len_in = len;
+	
 	for (bucle=0;bucle<len_in;bucle++)
 	{
 		if (st[bucle]=='&')
@@ -523,9 +525,7 @@ guint gMnemonic_correctMarkup(char *st,char **buf)
 
 void gMnemonic_returnText(char *st,char **buf)
 {
-	long bucle,b2;
-	long len=strlen(st);
-	long len_in=len;
+	int bucle,b2;
 	
 	if (!st || !*st)
 	{
@@ -533,6 +533,9 @@ void gMnemonic_returnText(char *st,char **buf)
 		return;
 	}
 		
+	int len = strlen(st);
+	int len_in = len;
+	
 	for (bucle=0;bucle<len_in;bucle++)
 	{
 		if (st[bucle]=='_')
@@ -735,7 +738,7 @@ static void add_space(GString *str)
 		g_string_append_c(str, ' ');
 }
 
-char *gt_html_to_pango_string(char *html, int len_html, bool newline)
+char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 {
 	static const char *title[] =
 	{
@@ -750,27 +753,20 @@ char *gt_html_to_pango_string(char *html, int len_html, bool newline)
 	
 	static const char *accept[] = { "b", "big", "i", "s", "sub", "sup", "small", "tt", "u", NULL };
 	
-	static const char *size[] = 
-	{
-		"\"7\"", "xx-large",
-		"\"6\"", "x-large",
-		"\"5\"", "large",
-		"\"4\"", "medium",
-		"\"3\"", "small",
-		"\"2\"", "x-small",
-		"\"1\"", "xx-small",
-		"\"+1\"", "larger",
-		"\"-1\"", "smaller",
-		NULL, NULL
-	};
+	static const char *size_name[] = { "xx-small", "x-small", "small", "medium", "large", "x-large", "xx-large" };
+	static const int size_stack_len = 16;
 	
 	GString *pango = g_string_new("");
 	const char *p, *p_end, *p_markup;
 	char c;
 	char *token, *markup, **attr;
+	const char *pp;
 	gsize len;
 	bool end_token = false;
-	const char **pt, **ps;
+	const char **pt;
+	int size = 3; // medium
+	char size_stack[size_stack_len];
+	int size_stack_ptr = 0;
 	
 	p_end = &html[len_html < 0 ? strlen(html) : len_html];
 	p_markup = NULL;
@@ -861,9 +857,21 @@ char *gt_html_to_pango_string(char *html, int len_html, bool newline)
 			if (!strcasecmp(token, "font"))
 			{
 				if (end_token)
+				{
+					if (size_stack_ptr > 0)
+					{
+						size_stack_ptr--;
+						if (size_stack_ptr < size_stack_len)
+							size = size_stack[size_stack_ptr];
+					}
 					g_string_append(pango, "</span>");
+				}
 				else
 				{
+					if (size_stack_ptr < size_stack_len)
+						size_stack[size_stack_ptr] = size;
+					size_stack_ptr++;
+
 					g_string_append(pango, "<span");
 					for (pt = (const char **)attr; *pt; pt++)
 					{
@@ -880,16 +888,33 @@ char *gt_html_to_pango_string(char *html, int len_html, bool newline)
 						else if (!strncasecmp(*pt, "size=", 5))
 						{
 							g_string_append(pango, " size=");
-							for (ps = size; *ps; ps += 2)
+							pp = *pt + 6;
+							if (*pp == '"')
+								pp++;
+							if (isdigit(*pp))
 							{
-								if (!strcasecmp(*ps, *pt + 5))
-								{
-									g_string_append_printf(pango, "\"%s\"", ps[1]);
-									break;
-								}
+								size = *pp - '1';
+								pp++;
 							}
-							if (!*ps)
-								g_string_append(pango, *pt + 5);
+							else if (*pp == '+' && isdigit(pp[1]))
+							{
+								size += p[1] - '0';
+								pp += 2;
+							}
+							else if (*pp == '-' && isdigit(pp[1]))
+							{
+								size -= pp[1] - '0';
+								pp += 2;
+							}
+							
+							if (size < 0)
+								size = 0;
+							else if (size > 6)
+								size = 6;
+							
+							g_string_append(pango, "\"");
+							g_string_append(pango, size_name[size]);
+							g_string_append(pango, "\"");
 						}
 					}
 					g_string_append(pango, ">");
@@ -945,18 +970,30 @@ char *gt_html_to_pango_string(char *html, int len_html, bool newline)
 				
 			entity_len = p - entity_start;
 			if (*p != ';')
-				p--;
+				entity_len = 0;
 				
 			if (entity_len > 1)
 			{
-				const struct entity *e = kde_findEntity(entity_start, entity_len);
-				if (e)
+				if ((entity_len == 3 && !strncasecmp(entity_start, "amp", 3))
+					  || (entity_len == 2 && (!strncasecmp(entity_start, "lt", 2) || !strncasecmp(entity_start, "gt", 2))))
 				{
-					entity_unicode = e->code;
-					g_string_append_unichar(pango, entity_unicode);
+					g_string_append_len(pango, entity_start - 1, entity_len + 2);
+					continue;
+				}
+				else
+				{
+					const struct entity *e = kde_findEntity(entity_start, entity_len);
+					if (e)
+					{
+						entity_unicode = e->code;
+						g_string_append_unichar(pango, entity_unicode);
+						continue;
+					}
 				}
 			}
 			
+			g_string_append(pango, "&amp;");
+			p = entity_start - 1;
 			continue;
 		}
 	
