@@ -447,6 +447,12 @@ int libsmtp_int_nextpart (struct libsmtp_session_struct *libsmtp_session)
 						if (libsmtp_int_send (libsmtp_temp_gstring, libsmtp_session, 1))
 							return LIBSMTP_ERRORSENDFATAL;
 					}
+
+					/* Adds a blank line */
+					g_string_assign (libsmtp_temp_gstring, "\r\n");
+					
+					if (libsmtp_int_send(libsmtp_temp_gstring, libsmtp_session, 1))
+						return LIBSMTP_ERRORSENDFATAL;
         }
       }
 
@@ -679,7 +685,7 @@ int libsmtp_int_check_part (struct libsmtp_part_struct *libsmtp_int_part)
 
 const char *libsmtp_int_lookup_mime_type (struct libsmtp_part_struct *libsmtp_int_part)
 {
-  if ((libsmtp_int_part->Type >= 0) && (libsmtp_int_part->Type < LIBSMTP_MAX_MIME))
+  if ((libsmtp_int_part->Type >= 0) && (libsmtp_int_part->Type <= LIBSMTP_MAX_MIME))
   {
     if (libsmtp_int_part->Type == LIBSMTP_MIME_CUSTOM)
       return libsmtp_int_part->CustomType->str;
