@@ -22,7 +22,6 @@
 
 ***************************************************************************/
 
-
 #define __CTEXTBOX_CPP
 
 #include <qapplication.h>
@@ -333,15 +332,30 @@ END_METHOD
 #undef THIS
 #define THIS OBJECT(CCOMBOBOX)
 
-static void setCurrentItem(void *_object, int item)
+static void combo_raise_click(void *_object)
 {
-	if (item == COMBOBOX->currentItem())
+	if (THIS->click)
 		return;
+	THIS->click = true;
+	GB.Raise(THIS, EVENT_Click, 0);
+	THIS->click = false;
+}
+
+static int combo_get_current_item(void *_object)
+{
+	return COMBOBOX->count() == 0 ? -1 : COMBOBOX->currentItem();
+}
+
+static void combo_set_current_item(void *_object, int item)
+{
+	if (item != combo_get_current_item(THIS))
+	{
+		if (item < COMBOBOX->count())
+			COMBOBOX->setCurrentItem(item);
+	}
 	
-  if (item < COMBOBOX->count())
-  	COMBOBOX->setCurrentItem(item);
-  if (item >= 0)
-    GB.Raise(_object, EVENT_Click, 0);
+  if (item >= 0 && !COMBOBOX->signalsBlocked())
+		combo_raise_click(THIS);
 }
 
 
@@ -377,6 +391,64 @@ static void combo_set_editable(void *_object, bool ed)
   COMBOBOX->calcMinimumHeight();
 }
 
+static int combo_find_item(void *_object, const QString& s)
+{
+  for (int i = 0; i < (int)COMBOBOX->count(); i++)
+  {
+    if (COMBOBOX->text(i) == s)
+      return i;
+  }
+
+  return (-1);
+}
+
+static void combo_set_text(CCOMBOBOX *_object, QString &text)
+{
+	int pos;
+	
+	if (COMBOBOX->editable())
+		COMBOBOX->lineEdit()->setText(text);
+
+	pos = combo_find_item(THIS, text);
+	//COMBOBOX->blockSignals(true);
+	combo_set_current_item(_object, pos);
+}
+
+static void combo_get_list(void *_object, GB_ARRAY array)
+{
+	int i;
+	char *str;
+	
+	for (i = 0; i < COMBOBOX->count(); i++)
+	{
+		GB.NewString(&str, TO_UTF8(COMBOBOX->text(i)), 0);
+		*((char **)GB.Array.Get(array, i)) = str;
+	}
+}
+
+static void combo_set_list(void *_object, GB_ARRAY array)
+{
+	int i;
+	QString text = COMBOBOX->currentText();
+	
+  COMBOBOX->blockSignals(true);
+  COMBOBOX->clear();
+
+	if (array)
+	{
+		for (i = 0; i < GB.Array.Count(array); i++)
+		{
+			COMBOBOX->insertItem(TO_QSTRING(*((char **)GB.Array.Get(array, i))));
+		}
+	}
+
+	if (THIS->sorted)
+		COMBOBOX->listBox()->sort();
+
+	combo_set_text(THIS, text);
+	
+  COMBOBOX->blockSignals(false);
+}
 
 
 BEGIN_METHOD(CCOMBOBOX_new, GB_OBJECT parent)
@@ -414,19 +486,12 @@ END_METHOD
 
 BEGIN_PROPERTY(CCOMBOBOX_text)
 
-  int pos;
-
   if (READ_PROPERTY)
     GB.ReturnNewZeroString(TO_UTF8(COMBOBOX->currentText()));
   else
   {
     QString text = QSTRING_PROP();
-
-    if (COMBOBOX->editable())
-      COMBOBOX->lineEdit()->setText(text);
-
-    pos = CTextBox::find(COMBOBOX, text);
-    setCurrentItem(_object, pos);
+		combo_set_text(THIS, text);
   }
 
 END_PROPERTY
@@ -487,16 +552,23 @@ BEGIN_METHOD(CCOMBOBOX_add, GB_STRING item; GB_INTEGER pos)
 	int pos;
 
   COMBOBOX->blockSignals(true);
-  index = COMBOBOX->currentItem();
+
+	index = combo_get_current_item(THIS);
+	
   pos = VARGOPT(pos, -1);
   if (pos < 0 || pos >= COMBOBOX->count())
   	pos = -1;
   COMBOBOX->insertItem(QSTRING_ARG(item), pos);
   if (THIS->sorted)
     COMBOBOX->listBox()->sort();
-  COMBOBOX->setCurrentItem(index);
+	
+	if (index >= 0)
+		combo_set_current_item(THIS, index);
+	else
+		combo_set_current_item(THIS, 0);
+	
   COMBOBOX->blockSignals(false);
-
+	
 END_METHOD
 
 
@@ -533,16 +605,16 @@ END_PROPERTY
 BEGIN_PROPERTY(CCOMBOBOX_index)
 
   if (READ_PROPERTY)
-    GB.ReturnInteger(COMBOBOX->currentItem());
+    GB.ReturnInteger(combo_get_current_item(THIS));
   else
-    setCurrentItem(_object, VPROP(GB_INTEGER));
+    combo_set_current_item(THIS, VPROP(GB_INTEGER));
 
 END_PROPERTY
 
 
 BEGIN_PROPERTY(CCOMBOBOX_current)
 
-  THIS->index = COMBOBOX->currentItem();
+  THIS->index = combo_get_current_item(THIS);
 
   if (THIS->index < 0)
     GB.ReturnNull();
@@ -570,7 +642,7 @@ END_METHOD
 
 BEGIN_METHOD(CCOMBOBOX_find, GB_STRING item)
 
-  GB.ReturnInteger(CTextBox::find(COMBOBOX, QSTRING_ARG(item)));
+  GB.ReturnInteger(combo_find_item(THIS, QSTRING_ARG(item)));
 
 END_METHOD
 
@@ -582,14 +654,12 @@ BEGIN_PROPERTY(CCOMBOBOX_list)
   if (READ_PROPERTY)
   {
   	GB.Array.New(&array, GB_T_STRING, COMBOBOX->count());
-    CTextBox::getAll(COMBOBOX, array);
+    combo_get_list(THIS, array);
     GB.ReturnObject(array);
 	}
   else
   {
-    CTextBox::setAll(COMBOBOX, (GB_ARRAY)VPROP(GB_OBJECT));
-    if (THIS->sorted)
-      COMBOBOX->listBox()->sort();
+    combo_set_list(THIS, (GB_ARRAY)VPROP(GB_OBJECT));
   }
 
 END_PROPERTY
@@ -625,6 +695,14 @@ void MyComboBox::calcMinimumHeight()
     setMinimumHeight(fm.lineSpacing() + 2);
 }
 
+void MyComboBox::hide()
+{
+	void *_object = CWidget::get(this);
+	
+	QComboBox::hide();
+	if (CWIDGET_active_control == (CWIDGET *)THIS)
+		focusNextPrevChild(true);
+}
 
 /***************************************************************************
 
@@ -648,53 +726,10 @@ void CTextBox::event_activate(void)
 
 void CTextBox::event_click()
 {
-  RAISE_EVENT(EVENT_Click);
+	GET_SENDER(_object);
+	combo_raise_click(THIS);
 }
 
-int CTextBox::find(QComboBox *list, const QString& s)
-{
-  for (int i = 0; i < (int)list->count(); i++)
-  {
-    if (list->text(i) == s)
-      return i;
-  }
-
-  return (-1);
-}
-
-
-void CTextBox::getAll(QComboBox *list, GB_ARRAY array)
-{
-	int i;
-	char *str;
-	
-	for (i = 0; i < list->count(); i++)
-	{
-		GB.NewString(&str, TO_UTF8(list->text(i)), 0);
-		*((char **)GB.Array.Get(array, i)) = str;
-	}
-}
-
-
-void CTextBox::setAll(QComboBox *list, GB_ARRAY array)
-{
-	int i;
-	
-  list->clear();
-  list->blockSignals(true);
-
-	if (array)
-	{
-		for (i = 0; i < GB.Array.Count(array); i++)
-		{
-			list->insertItem(TO_QSTRING(*((char **)GB.Array.Get(array, i))));
-		}
-	}
-
-  list->blockSignals(false);
-}
-
-
 /***************************************************************************
 
   Descriptions
