@@ -72,6 +72,9 @@ GB_CLASS CLASS_Printer;
 static QMap<int, int> _x11_to_qt_keycode;
 #endif
 
+static bool _focus_change = false;
+static CWIDGET *_old_active_control = 0;
+
 /* Control class */
 
 
@@ -481,6 +484,8 @@ BEGIN_PROPERTY(CCONTROL_font)
     else
       WIDGET->setFont(*(font->font));
     
+		//qDebug("%s: setFont: %s -> %s", THIS->name, font->font->toString().latin1(), WIDGET->font().toString().latin1());
+		
     *(((CFONT *)THIS->font)->font) = WIDGET->font();
   }
 
@@ -1591,6 +1596,8 @@ void CWidget::destroy()
 	
 	if (CWIDGET_active_control == ob)
 		CWIDGET_active_control = NULL;
+	if (_old_active_control == ob)
+		_old_active_control = NULL;
 
 	set_name(ob, 0);
 
@@ -1619,17 +1626,55 @@ static void post_dblclick_event(void *control)
   GB.Unref(&control);
 }
 
-static void post_gotfocus_event(void *_object)
+// static void post_gotfocus_event(void *_object)
+// {
+// 	//qDebug("post_gotfocus_event: %p (%p)", THIS, CWIDGET_active_control);
+// 	if (CWIDGET_active_control == THIS)
+//   	GB.Raise(THIS, EVENT_GotFocus, 0);
+// 	else
+// 		qDebug("No GotFocus!");
+//   GB.Unref(&_object);
+// }
+// 
+// static void post_lostfocus_event(void *control)
+// {
+// 	//qDebug("post_lostfocus_event: %p (%p)", control, CWIDGET_active_control);
+// 	if (CWIDGET_active_control != control)
+// 		GB.Raise(control, EVENT_LostFocus, 0);
+// 	else
+// 		qDebug("No LostFocus!");
+//   GB.Unref(&control);
+// }
+
+static void post_focus_change(void *)
 {
-	if (WIDGET && WIDGET->hasFocus())
-  	GB.Raise(THIS, EVENT_GotFocus, 0);
-  GB.Unref(&_object);
+	CWIDGET *current;
+	
+	for(;;)
+	{
+		current = CWIDGET_active_control;
+		if (current == _old_active_control)
+			break;
+
+		if (_old_active_control)
+			GB.Raise(_old_active_control, EVENT_LostFocus, 0);
+		
+		_old_active_control = current;
+		
+		if (current)
+			GB.Raise(current, EVENT_GotFocus, 0);
+	}
+	
+	_focus_change = FALSE;
 }
 
-static void post_lostfocus_event(void *control)
+void CWIDGET_handle_focus_change()
 {
-  GB.Raise(control, EVENT_LostFocus, 0);
-  GB.Unref(&control);
+	if (_focus_change)
+		return;
+	
+	_focus_change = TRUE;
+	GB.Post((void (*)())post_focus_change, NULL);
 }
 
 bool CWidget::eventFilter(QObject *widget, QEvent *event)
@@ -1668,31 +1713,17 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
   }
   else if (type == QEvent::FocusIn)
   {
-  	if (GB.CanRaise(control, EVENT_GotFocus))
-  	{
-			if (control != CWIDGET_active_control)
-			{
-				CWIDGET_active_control = control;
-				GB.Ref(control);
-				GB.Post((void (*)())post_gotfocus_event, (intptr_t)control);
-			}
-		}
-		
+		//qDebug("FocusIn: %p %s (%p)", control, control->name, CWIDGET_active_control);
+		CWIDGET_active_control = control;
+		CWIDGET_handle_focus_change();
 		CWINDOW_activate(control);
     //GB.Raise(control, EVENT_GotFocus, 0);
   }
   else if (type == QEvent::FocusOut)
   {
-  	if (GB.CanRaise(control, EVENT_LostFocus))
-  	{
-			if (control == CWIDGET_active_control)
-			{
-				CWIDGET_active_control = NULL;
-				GB.Ref(control);
-				GB.Post((void (*)())post_lostfocus_event, (intptr_t)control);
-			}
-		}
-    //GB.Raise(control, EVENT_LostFocus, 0);
+		//qDebug("FocusOut: %p %s (%p)", control, control->name, CWIDGET_active_control);
+		CWIDGET_active_control = NULL;
+		CWIDGET_handle_focus_change();
   }
   else if (type == QEvent::ContextMenu)
   {
@@ -1998,7 +2029,8 @@ _ACCEL:
 	else if (type == QEvent::Drop)
 	{
   	//if (!CWIDGET_test_flag(control, WF_NO_DRAG))
-		CDRAG_drag_drop((QWidget *)widget, control, (QDropEvent *)event);
+		if (CDRAG_drag_drop((QWidget *)widget, control, (QDropEvent *)event))
+			return true;
 	}
 	else if (type == QEvent::DragLeave)
 	{
