@@ -40,9 +40,6 @@
 #include <dirent.h>
 
 #include <unistd.h>
-#ifdef __GNU_LIBRARY__
-#include <getopt.h>
-#endif
 
 #include "gb_common.h"
 #include "gb_error.h"
@@ -60,7 +57,7 @@
 #include "gbc_output.h"
 
 
-#ifdef __GNU_LIBRARY__
+#if HAVE_GETOPT_LONG
 static struct option Long_options[] =
 {
   { "debug", 0, NULL, 'g' },
@@ -94,13 +91,13 @@ static void get_arguments(int argc, char **argv)
 {
   const char *dir;
   int opt;
-  #ifdef __GNU_LIBRARY__
+  #if HAVE_GETOPT_LONG
   int index = 0;
   #endif
 
   for(;;)
   {
-    #ifdef __GNU_LIBRARY__
+    #if HAVE_GETOPT_LONG
       opt = getopt_long(argc, argv, "gvaVhtpmsr:", Long_options, &index);
     #else
       opt = getopt(argc, argv, "gvaVhtpmsr:");
@@ -156,7 +153,7 @@ static void get_arguments(int argc, char **argv)
           COPYRIGHT
           "Usage: gbc" GAMBAS_VERSION_STRING " [options] [<project directory>]\n\n"
           "Options:"
-          #ifdef __GNU_LIBRARY__
+          #if HAVE_GETOPT_LONG
           "\n"
           "  -g  --debug                add debugging information\n"
           "  -v  --verbose              verbose output\n"
@@ -221,6 +218,7 @@ static void get_arguments(int argc, char **argv)
 
 static void compile_file(const char *file)
 {
+	int i;
   time_t time_src, time_form, time_pot, time_output;
 
   COMPILE_begin(file, main_trans);
@@ -255,7 +253,12 @@ static void compile_file(const char *file)
   //JOB->class_file = main_class_file;
 
   if (JOB->verbose)
-    puts(JOB->name);
+  {
+  	putchar('\n');
+  	for (i = 1; i <= 9; i++)
+  		printf("--------");
+  	printf("\nCompiling %s...\n", FILE_get_name(JOB->name));
+	}
 
   COMPILE_load();
   FORM_do(main_public);
@@ -275,6 +278,7 @@ static void compile_file(const char *file)
   #endif
 
   OUTPUT_do(main_swap);
+  CLASS_export();
 
 _FIN:
   COMPILE_end();
@@ -371,6 +375,17 @@ int main(int argc, char **argv)
   {
     COMPILE_init();
 
+		// Remove information files if we are compiling everything
+		
+		if (main_compile_all)
+		{
+			if (main_verbose)
+				puts("Removing .info and .list files");
+			chdir(FILE_get_dir(COMP_project));
+			unlink(".info");
+			unlink(".list");
+		}
+
     init_files(FILE_get_dir(COMP_project));
 
 		for (i = 0; i < ARRAY_count(_files); i++)
