@@ -33,7 +33,7 @@ EXTERN sigjmp_buf CHECK_jump;
 PUBLIC void CHECK_enter(void);
 PUBLIC void CHECK_leave(void);
 PUBLIC bool CHECK_got_error(void);
-PUBLIC bool CHECK_address(void *ptr, int len);
-PUBLIC bool CHECK_strlen(char *ptr, int *len);
+PUBLIC bool CHECK_address(void *ptr, size_t len);
+PUBLIC bool CHECK_strlen(char *ptr, size_t *len);
 
 #endif
